unit ComPort;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, ExtCtrls, IdGlobal;

type
  TComportChangeEvent = procedure(Sender: TObject; Pin: string) of object;
  TComPort = class(TComponent)
  private
    TimerComPort : TTimer;
    dwPortHandle : DWORD;

    CTSStatus, OldCTSStatus : bool;
    DSRStatus, OldDSRStatus : bool;
    RingStatus, OldRingStatus : bool;
    RLSDStatus, OldRLSDStatus : bool;

    FComPort : String;
    FComPortInterval : integer;

    FDTRPower : bool;
    FRTSPower : bool;

//    FOnComPortCTSStatusChange : TNotifyEvent;
//    FOnComPortDSRStatusChange : TNotifyEvent;
//    FOnComPortRingStatusChange : TNotifyEvent;
//    FOnComPortRLSDStatusChange : TNotifyEvent;

    FOnComPortStartSnooping : TNotifyEvent;
    FOnComPortStopSnooping : TNotifyEvent;
    FOnComPortStatusChange : TComportChangeEvent;

    procedure Snoop(Sender: TObject);
    procedure ChangeComPort(port : string);
    procedure SetDTRPower(Power : bool);
    procedure SetRTSPower(Power : bool);
  protected
    { Protected declarations }
  public
    function CTSEnabled() : bool;
    function DSREnabled() : bool;
    function RingEnabled() : bool;
    function RLSDEnabled() : bool;
    constructor Create(AOwner : Tcomponent); override;
    destructor Destroy(); override;
    procedure StartSnooping();
    procedure StopSnooping();
  published
    property OnComPortStartSnooping : TNotifyEvent read FOnComPortStartSnooping write FOnComPortStartSnooping;
    property OnComPortStopSnooping : TNotifyEvent read FOnComPortStopSnooping write FOnComPortStopSnooping;

    property OnComPortStatusChange : TComportChangeEvent read FOnComPortStatusChange write FOnComPortStatusChange;
//    property OnComPortCTSStatusChange : TNotifyEvent read FOnComPortCTSStatusChange write FOnComPortCTSStatusChange;
//    property OnComPortDSRStatusChange : TNotifyEvent read FOnComPortDSRStatusChange write FOnComPortDSRStatusChange;
//    property OnComPortRingStatusChange : TNotifyEvent read FOnComPortRingStatusChange write FOnComPortRingStatusChange;
//    property OnComPortRLSDStatusChange : TNotifyEvent read FOnComPortRLSDStatusChange write FOnComPortRLSDStatusChange;

    property ComPort : String read FComPort write ChangeComPort;
    property ComPortInterval : integer read FComPortInterval write FComPortInterval;
    property DTRPower : bool read FDTRPower write SetDTRPower;
    property RTSPower : bool read FRTSPower write SetRTSPower;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Gestapo', [TComPort]);
end;

{ TComPort }

constructor TComPort.Create(AOwner: Tcomponent);
begin
  inherited;
  OnComPortStartSnooping := nil;
  OnComPortStopSnooping := nil;

  OnComPortStatusChange := nil;
//  OnComPortCTSStatusChange := nil;
//  OnComPortDSRStatusChange := nil;
//  OnComPortRingStatusChange := nil;
//  OnComPortRLSDStatusChange := nil;

  TimerComPort := TTimer.Create(self);
  TimerComPort.Enabled := False;
  TimerComPort.OnTimer := Snoop;

  ComPort := 'COM1';
  ComPortInterval := 100;

  DTRPower := false;
  RTSPower := false;

  OldCTSStatus := false;
  OldDSRStatus := false;
  OldRingStatus := false;
  OldRLSDStatus := false;
end;

destructor TComPort.Destroy;
begin
  CloseHandle(dwPortHandle);
  inherited;
end;

procedure TComPort.StartSnooping;
begin
  TimerComPort.Interval := ComPortInterval;
  DTRPower := true;
  TimerComPort.Enabled := True;
  if Assigned(OnComPortStartSnooping) Then
    OnComPortStartSnooping(self);
end;

procedure TComPort.StopSnooping;
begin
  DTRPower := false;
  TimerComPort.Enabled := False;
  if Assigned(OnComPortStopSnooping) Then
    OnComPortStopSnooping(self);
end;

procedure TComPort.Snoop(Sender: TObject);
var dwStatus : DWORD;
var bits : string;
begin

  GetCommModemStatus(dwPortHandle,dwStatus);
  bits := IntToBin(dwStatus shr 4);

  CTSStatus := bits[length(bits)-0] = '1';
  DSRStatus := bits[length(bits)-1] = '1';
  RingStatus := bits[length(bits)-2] = '1';
  RLSDStatus := bits[length(bits)-3] = '1';

  if (OldCTSStatus <> CTSStatus) Then
    if Assigned(OnComPortStatusChange) Then
      OnComPortStatusChange(self,'CTS');
  OldCTSStatus := bits[length(bits)-0] = '1';

  if (OldDSRStatus <> DSRStatus) Then
    if Assigned(OnComPortStatusChange) Then
      OnComPortStatusChange(self,'DSR');
  OldDSRStatus := bits[length(bits)-1] = '1';

  if (OldRingStatus <> RingStatus) Then
    if Assigned(OnComPortStatusChange) Then
      OnComPortStatusChange(self,'RING');
  OldRingStatus := bits[length(bits)-2] = '1';

  if (OldRLSDStatus <> RLSDStatus) Then
    if Assigned(OnComPortStatusChange) Then
      OnComPortStatusChange(self,'RLSD');
  OldRLSDStatus := bits[length(bits)-3] = '1';
end;

function TComPort.CTSEnabled : bool;
begin
  result := CTSStatus;
end;

function TComPort.DSREnabled: bool;
begin
  result := DSRStatus;
end;

function TComPort.RingEnabled: bool;
begin
  result := RingStatus;
end;

function TComPort.RLSDEnabled: bool;
begin
  result := RLSDStatus;
end;

procedure TComPort.ChangeComPort(port: string);
begin
  FComPort := port;
  CloseHandle(dwPortHandle);
  dwPortHandle := CreateFile(PChar(FComPort), GENERIC_READ or GENERIC_WRITE, 0, nil,
                             OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, LongInt(0));
end;

procedure TComPort.SetDTRPower(Power: bool);
begin
  if Power then
  begin
    EscapeCommFunction(dwPortHandle,SETDTR);
    FDTRPower := true;
  end
  else
  begin
    EscapeCommFunction(dwPortHandle,CLRDTR);
    FDTRPower := false;
  end;
end;

procedure TComPort.SetRTSPower(Power: bool);
begin
  if Power then
  begin
    EscapeCommFunction(dwPortHandle,SETRTS);
    FRTSPower := true;
  end
  else
  begin
    EscapeCommFunction(dwPortHandle,CLRRTS);
    FRTSPower := false;
  end;
end;

end.
