unit CommunicationClient;

interface

uses
  Windows, Messages, QDialogs, SysUtils, Classes, ScktComp, IdTrivialFTP, helptools;

var ReceiveBuffer : AnsiString;

type
  TReadTextEvent = procedure(Sender: TObject; Text: string) of object;
  TReadFileEvent = procedure(Sender: TObject; Filename: string) of object;

type
  TCommunicationClient = class(TClientSocket)
  private
    FTPClient : TIdTrivialFTP;
    FOnReadFile: TReadFileEvent;
    FOnReadText: TReadTextEvent;
    FName: string;
    FHostIP: string;
    procedure SetHostIP(NewHostIP : string);
    procedure ReadData(Sender: TObject; Socket: TCustomWinSocket; SocketEvent: TSocketEvent);
  protected

  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Start();
    procedure SendText(Text: string);
    procedure SendFile(localname : string; remotename : string);
    procedure GetFile(remotename : string; localname : string);
  published
    property Port;
    property HostIP : string read FHostIP write SetHostIP;
    property Name : string read FName write FName;
    property OnReadText : TReadTextEvent read FOnReadText write FOnReadText;
    property OnReadFile : TReadFileEvent read FOnReadFile write FOnReadFile;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Gestapo', [TCommunicationClient]);
end;

{ TCommunicationClient }

constructor TCommunicationClient.Create(AOwner: TComponent);
begin
  inherited;
  Host := '127.0.0.1';   // LocalHost
  Port := 10079;
  Name := 'ClientName';
  Active := false;
  socket.OnSocketEvent := ReadData;
  ClientType := ctNonBlocking;
//  ClientType := ctBlocking;
  FTPClient := TIdTrivialFTP.Create(self);
  FTPClient.Host := '127.0.0.1';   // LocalHost
  FTPClient.Port := 69;
end;

destructor TCommunicationClient.Destroy;
begin
  Socket.SendText('CS_REMOVE_NAME ' + Name + '#');  // Remove identification name from server
  Close;                                   // close connection
  FTPClient.Active := false;               // close ftp connection
  FTPClient.Free;                          // destroy FTP component
  inherited;
end;

procedure TCommunicationClient.ReadData(Sender: TObject;
                                        Socket: TCustomWinSocket;
                                        SocketEvent: TSocketEvent);
var
  SenderAddress : string;
  ReceivedString : string;
//  ReceiveBuffer : string;
  Command : string;
  RemoteName : string;
  Localname : string;
//  counter : integer;
begin
  SenderAddress := Socket.RemoteAddress;
//  ReceivedString := Socket.ReceiveText;
//  counter := 0;
  ReceiveBuffer := ReceiveBuffer + Socket.ReceiveText;
  while true do//counter < 2 do  // this is taken from hat
  begin
//    counter := counter + 1;
    ReceivedString := ''; //needed if while loop is runned more than once
    ReceivedString := ParseString(ReceiveBuffer,'#');
    if ReceivedString = '' then break;    // exit from while loop
    Command := '';
    Command := ParseString(ReceivedString);

    if Command = 'CC_TEXT' then
      if Assigned(OnReadText) Then
        OnReadText(self,ReceivedString);
    if Command = 'CC_FILE' then
      if Assigned(OnReadFile) Then
        OnReadFile(self,ReceivedString);
    if Command = 'CC_RECEIVE' then
    begin
      RemoteName := ParseString(ReceivedString);
      Localname := ReceivedString;
      GetFile(RemoteName, LocalName);
      if Assigned(OnReadFile) Then
        OnReadFile(self,Localname);
    end;
    if Command = 'CC_SEND' then
    begin
      Localname := ParseString(ReceivedString);
      RemoteName := ReceivedString;
      SendFile(LocalName, RemoteName);
    end;
    if Command = 'CC_ADD_NAME' then
    begin
      Socket.SendText('CS_ADD_NAME ' + ReceivedString + ' ' + Name + '#');   // ConnectionNumber + Identification Name
    end;
    if Command = 'CC_REMOVE_NAME' then
    begin
      Socket.SendText('CS_REMOVE_NAME ' + Name + '#');   // Remove identification name from server
    end;

//    if ReceiveBuffer = '' then exit;  // full command string has received
  end;  // end while loop
end;

procedure TCommunicationClient.GetFile(remotename : string; localname : string);
begin
  if localname = '' then exit;
  if remotename = '' then exit;
  FTPClient.Get(remotename, localname);
//  sleep(100);
end;

procedure TCommunicationClient.SendFile(localname : string; remotename : string);
begin
  if localname = '' then exit;
  if remotename = '' then exit;
  FTPClient.Put(localname, remotename);
//  sleep(100);
  Socket.SendText('CS_FILE ' + Name + ' ' + remotename + '#');
end;

procedure TCommunicationClient.Start;
begin
  Close;
  Open;
  FTPClient.Active := true;
end;

procedure TCommunicationClient.SendText(Text: string);
begin
  Socket.SendText('CS_TEXT ' + Name + ' ' + Text + '#');
end;

procedure TCommunicationClient.SetHostIP(NewHostIP: string);
begin
  FHostIP := NewHostIP;
  Host := NewHostIP;
  FTPClient.Host := NewHostIP;
end;

end.
