unit DeviceController;

interface

uses
  Windows, Messages, SysUtils, Classes, Cameras, ComPort, Joystick, mmsystem,
  QExtCtrls;

type
  TPictureGrappedEvent = procedure(Sender: string) of object;
  TValueChangeEvent = procedure(Sender: string; Value : string) of object;
  TStatusChangeEvent = procedure(Sender : string; Value : string) of object;

type

  TDeviceController = class(TComponent)
  private
    EventCounter : integer;
    EventTimer : TTimer;
    FOnWetDetect: TStatusChangeEvent;
    FOnMotionDetect: TStatusChangeEvent;
    FOnOnOffDetect: TStatusChangeEvent;
    FOnTemperatureDetect: TValueChangeEvent;
    FOnPictureGrapped: TPictureGrappedEvent;
    procedure EventHandler(Sender: TObject);
    procedure EventRaiser(EventType : string; Sender : string; value : string = '');
  protected

  public
    Joystick1 : TJoystick;
    joystick1xoldvalue : integer;
    joystick1xmin : integer;
    joystick1xminc : integer;
    joystick1xFactor : integer;
    joystick1xenabled : bool;
    joystick1xInterval : integer;
    joystick1yoldvalue : integer;
    joystick1ymin : integer;
    joystick1yminc : integer;
    joystick1yFactor : integer;
    joystick1yenabled : bool;
    joystick1yInterval : integer;

    Joystick2 : TJoystick;
    joystick2xoldvalue : integer;
    joystick2xmin : integer;
    joystick2xminc : integer;
    joystick2xFactor : integer;
    joystick2xenabled : bool;
    joystick2xInterval : integer;
    joystick2yoldvalue : integer;
    joystick2ymin : integer;
    joystick2yminc : integer;
    joystick2yFactor : integer;
    joystick2yenabled : bool;
    joystick2yInterval : integer;

    ComPort1 : TComPort;
    ComPort1CTSType : string;
    ComPort1CTSStatusDefault : bool;
    Comport1CTSInterval : integer;
    ComPort1DSRType : string;
    ComPort1DSRStatusDefault : bool;
    Comport1DSRInterval : integer;
    ComPort1RingType : string;
    ComPort1RingStatusDefault : bool;
    Comport1RingInterval : integer;
    ComPort1RLSDType : string;
    ComPort1RLSDStatusDefault : bool;
    Comport1RLSDInterval : integer;
    ComPort1RTSType : string;
    ComPort1RTSStatusDefault : bool;
    Comport1RTSInterval : integer;

    ComPort2 : TComPort;
    ComPort2CTSType : string;
    ComPort2CTSStatusDefault : bool;
    Comport2CTSInterval : integer;
    ComPort2DSRType : string;
    ComPort2DSRStatusDefault : bool;
    Comport2DSRInterval : integer;
    ComPort2RingType : string;
    ComPort2RingStatusDefault : bool;
    Comport2RingInterval : integer;
    ComPort2RLSDType : string;
    ComPort2RLSDStatusDefault : bool;
    Comport2RLSDInterval : integer;
    ComPort2RTSType : string;
    ComPort2RTSStatusDefault : bool;
    Comport2RTSInterval : integer;

    Cameras : TCameras;
    Camera0Interval : integer;
    Camera1Interval : integer;
    Camera2Interval : integer;
    Camera3Interval : integer;

    Constructor Create(AOwner : TComponent); override;
    Destructor Destroy(); override;
    Procedure SearchDevices();
    Procedure StartSurvey();
    Procedure StopSurvey();
    Procedure Grab_Frame();

    Procedure ComPort1StatusChange(Sender : TObject; Pin: string);
    Procedure ComPort2StatusChange(Sender : TObject; Pin: string);

    procedure Joystick1XAxelChange(Sender: TObject);
    procedure Joystick1YAxelChange(Sender: TObject);
    procedure Joystick2XAxelChange(Sender: TObject);
    procedure Joystick2YAxelChange(Sender: TObject);
  published
    property OnMotionDetect : TStatusChangeEvent read FOnMotionDetect write FOnMotionDetect;
    property OnWetDetect : TStatusChangeEvent read FOnWetDetect write FOnWetDetect;
    property OnOnOffDetect : TStatusChangeEvent read FOnOnOffDetect write FOnOnOffDetect;
    property OnTemperatureDetect : TValueChangeEvent read FOnTemperatureDetect write FOnTemperatureDetect;
    property OnPictureGrapped : TPictureGrappedEvent read FOnPictureGrapped write FOnPictureGrapped;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Gestapo', [TDeviceController]);
end;

{ TDeviceController }

constructor TDeviceController.Create(AOwner: TComponent);
begin
  inherited;
  cameras := TCameras.Create(self);

  ComPort1 := TComport.Create(self);
  ComPort1.ComPort := 'COM1';
  ComPort1.ComPortInterval := 100;
  ComPort1.OnComPortStatusChange := ComPort1StatusChange;

  ComPort2 := TComport.Create(self);
  ComPort2.ComPort := 'COM2';
  ComPort2.ComPortInterval := 100;
  ComPort2.OnComPortStatusChange := ComPort2StatusChange;

  Joystick1 := TJoystick.Create(self);
  Joystick1.SetJoystickID(JOYSTICKID1);
  Joystick1.ChangeLimit := 0.10;
  Joystick1.OnXAxelChange := Joystick1XAxelChange;
  Joystick1.OnYAxelChange := Joystick1YAxelChange;

  Joystick2 := TJoystick.Create(self);
  Joystick2.SetJoystickID(JOYSTICKID2);
  Joystick2.ChangeLimit := 0.10;
  Joystick2.OnXAxelChange := Joystick2XAxelChange;
  Joystick2.OnYAxelChange := Joystick2YAxelChange;

  FOnMotionDetect := nil;
  FOnWetDetect := nil;
  FOnOnOffDetect := nil;
  FOnTemperatureDetect := nil;
  FOnPictureGrapped := nil;

  EventCounter := 0;
  EventTimer := TTimer.Create(self);
  EventTimer.Enabled := false;
  EventTimer.Interval := 1000;
  EventTimer.OnTimer := EventHandler;
end;

destructor TDeviceController.Destroy;
begin
  EventTimer.Free;
  Cameras.Free;
  ComPort1.Free;
  ComPort2.Free;
  Joystick1.Free;
  Joystick2.Free;
  inherited;
end;

procedure TDeviceController.EventHandler(Sender: TObject);
begin
  EventCounter := EventCounter + 1;
  if EventCounter > 32000 then EventCounter := 0;
end;

procedure TDeviceController.EventRaiser(EventType : string; Sender: string; value : string = '');
begin
  if EventType = 'Disabled' then exit;

  if EventType = 'Motion_Detector' then
    if Assigned(OnMotionDetect) Then
      OnMotionDetect(EventType + '_' + Sender, value);

  if EventType = 'Wet_Dry_Sensor' then
    if Assigned(OnWetDetect) Then
      OnWetDetect(EventType + '_' + Sender, value);

  if EventType = 'On_Off_Sensor' then
    if Assigned(OnOnOffDetect) Then
      OnOnOffDetect(EventType + '_' + Sender, value);

  if EventType = 'Temperature_Sensor' then
    if Assigned(OnTemperatureDetect) Then
      OnTemperatureDetect(EventType + '_' + Sender, value);

  if EventType = 'Camera' then
    if Assigned(OnPictureGrapped) Then
      OnPictureGrapped(EventType + '_' + Sender);

end;

procedure TDeviceController.ComPort1StatusChange(Sender: TObject; Pin: string);
begin
  if (pin = 'CTS') then
    if (Comport1.CTSEnabled <> ComPort1CTSStatusDefault) then EventRaiser(ComPort1CTSType,'com1'+Pin, 'on')
    else EventRaiser(ComPort1CTSType,'com1'+Pin, 'off');
  if (pin = 'DSR') then
    if (Comport1.DSREnabled <> ComPort1DSRStatusDefault) then EventRaiser(ComPort1DSRType,'com1'+Pin, 'on')
    else EventRaiser(ComPort1DSRType,'com1'+Pin, 'off');
  if (pin = 'RING') then
    if (Comport1.RingEnabled <> ComPort1RingStatusDefault) then EventRaiser(ComPort1RingType,'com1'+Pin, 'on')
    else EventRaiser(ComPort1RingType,'com1'+Pin, 'off');
  if (pin = 'RLSD') then
    if (Comport1.RLSDEnabled <> ComPort1RLSDStatusDefault) then EventRaiser(ComPort1RLSDType,'com1'+Pin, 'on')
    else EventRaiser(ComPort1RLSDType,'com1'+Pin, 'off');
end;

procedure TDeviceController.ComPort2StatusChange(Sender: TObject;
  Pin: string);
begin
  if (pin = 'CTS') then
    if (Comport2.CTSEnabled <> ComPort2CTSStatusDefault) then EventRaiser(ComPort2CTSType,'com2'+Pin, 'on')
    else EventRaiser(ComPort1CTSType,'com2'+Pin, 'off');
  if (pin = 'DSR') then
    if (Comport2.DSREnabled <> ComPort2DSRStatusDefault) then EventRaiser(ComPort2DSRType,'com2'+Pin, 'on')
    else EventRaiser(ComPort1DSRType,'com2'+Pin, 'off');
  if (pin = 'RING') then
    if (Comport2.RingEnabled <> ComPort2RingStatusDefault) then EventRaiser(ComPort2RingType,'com2'+Pin, 'on')
    else EventRaiser(ComPort1RingType,'com2'+Pin, 'off');
  if (pin = 'RLSD') then
    if (Comport2.RLSDEnabled <> ComPort2RLSDStatusDefault) then EventRaiser(ComPort2RLSDType,'com2'+Pin, 'on')
    else EventRaiser(ComPort1RLSDType,'com2'+Pin, 'off');
end;

procedure TDeviceController.Joystick1XAxelChange(Sender: TObject);
var value : integer;
begin
  if not(joystick1xenabled) then exit;
  value := Round((( integer(joystick1.GetJoystickPos.wXpos) - joystick1xmin) / joystick1xfactor) + joystick1xminC);
  if value <> joystick1xoldvalue then EventRaiser('Temperature_Sensor','joy1x',IntToStr(value));
end;

procedure TDeviceController.Joystick1YAxelChange(Sender: TObject);
var value : integer;
begin
  if not(joystick1yenabled) then exit;
  value := Round((( integer(joystick1.GetJoystickPos.wYpos) - joystick1ymin) / joystick1yfactor) + joystick1yminC);
  if value <> joystick1yoldvalue then EventRaiser('Temperature Sensor','joy1y',IntToStr(value));
end;

procedure TDeviceController.Joystick2XAxelChange(Sender: TObject);
var value : integer;
begin
  if not(joystick2xenabled) then exit;
  value := Round((( integer(joystick2.GetJoystickPos.wXpos) - joystick2xmin) / joystick2xfactor) + joystick2xminC);
  if value <> joystick2xoldvalue then EventRaiser('Temperature Sensor','joy2x',IntToStr(value));
end;

procedure TDeviceController.Joystick2YAxelChange(Sender: TObject);
var value : integer;
begin
  if not(joystick2yenabled) then exit;
  value := Round((( integer(joystick2.GetJoystickPos.wYpos) - joystick2ymin) / joystick2yfactor) + joystick2yminC);
  if value <> joystick2yoldvalue then EventRaiser('Temperature Sensor','joy2y',IntToStr(value));
end;

procedure TDeviceController.Grab_Frame;
begin
  cameras.GRAB_FRAME;
  EventRaiser('Camera','cam' + IntToStr(cameras.selected_camera));
end;

procedure TDeviceController.SearchDevices;
begin
  Cameras.SearchCameras;
end;

procedure TDeviceController.StartSurvey;
begin
  if (ComPort1CTSType <> 'Disabled') or
     (ComPort1DSRType <> 'Disabled') or
     (ComPort1RingType <> 'Disabled') or
     (ComPort1RLSDType <> 'Disabled') or
     (ComPort1RTSType <> 'Disabled') then ComPort1.StartSnooping;

  if (ComPort2CTSType <> 'Disabled') or
     (ComPort2DSRType <> 'Disabled') or
     (ComPort2RingType <> 'Disabled') or
     (ComPort2RLSDType <> 'Disabled') or
     (ComPort2RTSType <> 'Disabled') then ComPort2.StartSnooping;

  if joystick1xenabled or
     joystick1yenabled then Joystick1.StartSnooping;

  if joystick2xenabled or
     joystick2yenabled then Joystick2.StartSnooping;
end;

procedure TDeviceController.StopSurvey;
begin
  ComPort1.StopSnooping;
  ComPort2.StopSnooping;
  Joystick1.StopSnooping;
  Joystick2.StopSnooping;
end;

end.

