unit Joystick;

interface

uses
  Windows, Messages, SysUtils, Classes,mmsystem,ExtCtrls;

type
  TJoystick = class(TComponent)
  private
    TimerSnoop : TTimer;

    ID : cardinal;
    XAxelPos : integer;
    YAxelPos : integer;
    OldXAxelPos : integer;
    OldYAxelPos : integer;

    FOnXAxelChange : TNotifyEvent;
    FOnYAxelChange : TNotifyEvent;

    FChangeLimit : real;
    FJoystickInterval : integer;
    procedure SetChangeLimit(value : real);
    procedure SetJoystickInterval(value : integer);
  protected
    { Protected declarations }
  public
    constructor Create(AOwner : TComponent); override;
    function GetJoystickPos() : JOYINFO;
    procedure SetJoystickID(JoyID : cardinal);
    procedure StartSnooping();
    procedure StopSnooping();
    procedure Snoop(Sender : TObject);
  published
    property OnXAxelChange : TNotifyEvent read FOnXAxelChange write FOnXAxelChange;
    property OnYAxelChange : TNotifyEvent read FOnYAxelChange write FOnYAxelChange;
    property ChangeLimit : real read FChangeLimit write SetChangeLimit;
    property JoystickInterval : integer read FJoystickInterval write SetJoystickInterval;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Gestapo', [TJoystick]);
end;

{ TJoystick }

constructor TJoystick.Create(AOwner: TComponent);
begin
  inherited;
  OnXAxelChange := nil;
  OnYAxelChange := nil;

  TimerSnoop := TTimer.Create(self);
  TimerSnoop.Enabled := false;
  JoystickInterval := 100;
  TimerSnoop.OnTimer := Snoop;
  Changelimit := 0.05;
end;

function TJoystick.GetJoystickPos: JOYINFO;
var JoyInf : JOYINFO;
begin
  joyGetPos(ID,@JoyInf);
  result := JoyInf;
end;

procedure TJoystick.SetChangeLimit(value: real);
begin
  if (value >= 0) or (value <= 1) then
    FChangeLimit := value;
end;

procedure TJoystick.SetJoystickID(JoyID : cardinal);
begin
  ID := JoyID;
end;

procedure TJoystick.SetJoystickInterval(value: integer);
begin
  FJoystickInterval := value;
  TimerSnoop.Interval := FJoystickInterval;
end;

procedure TJoystick.Snoop(Sender : TObject);
var JoyInf : JOYINFO;
begin
  joyGetPos(ID,@JoyInf);
  XAxelPos := JoyInf.wXpos;
  YAxelPos := JoyInf.wYpos;
  if Abs(XAxelPos - OldXAxelPos) > XAxelPos*Changelimit Then
  begin
    if Assigned(OnXAxelChange) Then
      OnXAxelChange(self);
    OldXAxelPos := XAxelPos;
  end;

  if Abs(YAxelPos - OldYAxelPos) > YAxelPos*Changelimit Then
  begin
    if Assigned(OnYAxelChange) Then
      OnYAxelChange(self);
    OldYAxelPos := YAxelPos;
  end;

end;

procedure TJoystick.StartSnooping;
begin
  TimerSnoop.Enabled := true;

  OldXAxelPos := XAxelPos;
  OldYAxelPos := YAxelPos;
end;

procedure TJoystick.StopSnooping;
begin
  TimerSnoop.Enabled := false;
end;

end.
