unit SurveillanceController;

interface

uses
  Windows, Messages, SysUtils, Classes, CommunicationClient, DeviceController,
  QDialogs, ScktComp, IniFiles, helptools, ExtCtrls;

type
  TStatusEvent = procedure(Status : string) of object;

type
  TDevice = record
    DeviceName : string;
    DeviceType : string;
    LimitsStart : TDateTime;
    LimitsEnd : TDateTime;
    LimitsMin : string;
    LimitsMax : string;
  end;

type
  TSurveillanceController = class(TComponent)
  private
    ErrorCount : integer;
    DeviceController : TDeviceController;
    Client : TCommunicationClient;
    FIDName : string;
    FOnStatus: TStatusEvent;
    CameraTimer : TTimer;

    Devices : Array[0..17] of TDevice;

    Comport1CTSLimitsStart : TDateTime;
    Comport1CTSLimitsEnd : TDateTime;
    Comport1CTSLimitsMin : string;
    Comport1CTSLimitsMax : string;

    Comport1DSRLimitsStart : TDateTime;
    Comport1DSRLimitsEnd : TDateTime;
    Comport1DSRLimitsMin : string;
    Comport1DSRLimitsMax : string;

    Comport1RingLimitsStart : TDateTime;
    Comport1RingLimitsEnd : TDateTime;
    Comport1RingLimitsMin : string;
    Comport1RingLimitsMax : string;

    Comport1RLSDLimitsStart : TDateTime;
    Comport1RLSDLimitsEnd : TDateTime;
    Comport1RLSDLimitsMin : string;
    Comport1RLSDLimitsMax : string;

    Comport1RTSLimitsStart : TDateTime;
    Comport1RTSLimitsEnd : TDateTime;
    Comport1RTSLimitsMin : string;
    Comport1RTSLimitsMax : string;

    Comport2CTSLimitsStart : TDateTime;
    Comport2CTSLimitsEnd : TDateTime;
    Comport2CTSLimitsMin : string;
    Comport2CTSLimitsMax : string;

    Comport2DSRLimitsStart : TDateTime;
    Comport2DSRLimitsEnd : TDateTime;
    Comport2DSRLimitsMin : string;
    Comport2DSRLimitsMax : string;

    Comport2RingLimitsStart : TDateTime;
    Comport2RingLimitsEnd : TDateTime;
    Comport2RingLimitsMin : string;
    Comport2RingLimitsMax : string;

    Comport2RLSDLimitsStart : TDateTime;
    Comport2RLSDLimitsEnd : TDateTime;
    Comport2RLSDLimitsMin : string;
    Comport2RLSDLimitsMax : string;

    Comport2RTSLimitsStart : TDateTime;
    Comport2RTSLimitsEnd : TDateTime;
    Comport2RTSLimitsMin : string;
    Comport2RTSLimitsMax : string;

    Joystick1xLimitsStart : TDateTime;
    Joystick1xLimitsEnd : TDateTime;
    Joystick1xLimitsMin : string;
    Joystick1xLimitsMax : string;

    Joystick1yLimitsStart : TDateTime;
    Joystick1yLimitsEnd : TDateTime;
    Joystick1yLimitsMin : string;
    Joystick1yLimitsMax : string;

    Joystick2xLimitsStart : TDateTime;
    Joystick2xLimitsEnd : TDateTime;
    Joystick2xLimitsMin : string;
    Joystick2xLimitsMax : string;

    Joystick2yLimitsStart : TDateTime;
    Joystick2yLimitsEnd : TDateTime;
    Joystick2yLimitsMin : string;
    Joystick2yLimitsMax : string;

    procedure ClientError(Sender: TObject; Socket: TCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    procedure MotionDetect(Sender : string; Value: string);
    procedure WetDetect(Sender : string; Value: string);
    procedure OnOffDetect(Sender : string; Value: string);
    procedure TemperatureDetect(Sender: string; Value : string);
    procedure PictureGrapped(Sender : string);
    procedure GrabPicture(Sender : TObject);
    procedure SetIDName(NewName : string);
  protected

  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy(); override;
    procedure Prepare;
    procedure LoadSettings(filename: string = '.\localconfig.ini');
    procedure Start;
    procedure Stop;
  published
    property IDName : string read FIDName write SetIDName;
    property OnStatus : TStatusEvent read FOnStatus write FOnStatus;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Gestapo', [TSurveillanceController]);
end;

{ TSurveillanceController }

constructor TSurveillanceController.Create(AOwner: TComponent);
begin
  inherited;
  ErrorCount := 0;
  client := TCommunicationClient.Create(self);
  client.Port := 10079;
  client.HostIP := '127.0.0.1';   // LocalHost
  client.Name := 'ClientName';
  client.OnError := ClientError;
  DeviceController := TDeviceController.Create(self);
  DeviceController.OnMotionDetect := MotionDetect;
  DeviceController.OnWetDetect := WetDetect;
  DeviceController.OnOnOffDetect := OnOffDetect;
  DeviceController.OnTemperatureDetect := TemperatureDetect;
  DeviceController.OnPictureGrapped := PictureGrapped;
  //Devices
  CameraTimer := TTimer.Create(self);
  CameraTimer.Interval := 3600000;
  CameraTimer.OnTimer := GrabPicture;
  CameraTimer.Enabled := false;
  OnStatus := nil;
end;

destructor TSurveillanceController.Destroy;
begin
  CameraTimer.Free;
  DeviceController.Free;
  client.Free;
  inherited;
end;

procedure TSurveillanceController.ClientError(Sender: TObject;
  Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
  var ErrorCode: Integer);
begin
  ErrorCount := ErrorCount + 1;
  if ErrorCount > 10 then exit;
  if (ErrorEvent = eeAccept) or (ErrorEvent = eeConnect) then
  begin
    client.Start;
  end;
  ErrorCode := 0;
end;

procedure TSurveillanceController.MotionDetect(Sender : string; Value: string);
var FileName, CamName, bmpfile, datehelp, timehelp, help : string;
begin
  datehelp := DateToStr(Date);
  timehelp := TimeToStr(Time);
  //help := datehelp;
  //bmpfile := ParseString(help,'.',1) + '_';           // year
  //bmpfile := bmpfile + ParseString(help,'.',1) + '_'; // month
  //bmpfile := bmpfile + help + '_';                    // day
  //help := timehelp;
  //bmpfile := bmpfile + ParseString(help,':') + '_';   // hour
  //bmpfile := bmpfile + ParseString(help,':') + '_';   // min
  //bmpfile := bmpfile + help + '.bmp';                 // sec
  //CamName := 'Camera_Cam' + IntToStr(DeviceController.Cameras.selected_camera);
  //FileName := '.\log\' + IDName + '\' + CamName + '\' + bmpfile;
  client.SendText('DU_SAVELOG ' + Sender + ' ' + datehelp + ' ' + timehelp + ' ' + value + ' 1');
  if value = 'on' then begin
    //if self.
    client.SendText('DU_ALARM ' + datehelp + ' ' + timehelp + ' ' + IDName + ' ' + Sender + ' ' + value + ' 1');
    //DeviceController.Grab_Frame;
    //CameraTimer.Enabled := false;
    CameraTimer.Interval := 2000;
    //CameraTimer.Enabled := true;
  end
  else begin
    //CameraTimer.Enabled := false;
    if DeviceController.Camera0Interval*1000 >= 10000 then
      CameraTimer.Interval := DeviceController.Camera0Interval*1000;
    //CameraTimer.Enabled := true;
  end;
end;

procedure TSurveillanceController.WetDetect(Sender : string; Value: string);
var datehelp, timehelp : string;
begin
  datehelp := DateToStr(Date);
  timehelp := TimeToStr(Time);
  client.SendText('DU_SAVELOG ' + Sender + ' ' + datehelp + ' ' + timehelp + ' ' + value + ' 1');
  client.SendText('DU_ALARM ' + datehelp + ' ' + timehelp + ' ' + IDName + ' ' + Sender + ' ' + value + ' 1');
end;

procedure TSurveillanceController.OnOffDetect(Sender : string; Value: string);
var datehelp, timehelp : string;
begin
  datehelp := DateToStr(Date);
  timehelp := TimeToStr(Time);
  client.SendText('DU_SAVELOG ' + Sender + ' ' + datehelp + ' ' + timehelp + ' ' + value + ' 1');
  client.SendText('DU_ALARM ' + datehelp + ' ' + timehelp + ' ' + IDName + ' ' + Sender + ' ' + value + ' 1');
end;

procedure TSurveillanceController.TemperatureDetect(Sender: string; Value: string);
var datehelp, timehelp : string;
begin
  datehelp := DateToStr(Date);
  timehelp := TimeToStr(Time);
  client.SendText('DU_SAVELOG ' + Sender + ' ' + datehelp + ' ' + timehelp + ' ' + Value + ' 1');
end;

procedure TSurveillanceController.PictureGrapped(Sender: string);
var FileName, CamName, bmpfile, datehelp, timehelp, help : string;
begin
  datehelp := DateToStr(Date);
  timehelp := TimeToStr(Time);
  help := datehelp;
  bmpfile := ParseString(help,'.',1) + '_';           // year
  bmpfile := bmpfile + ParseString(help,'.',1) + '_'; // month
  bmpfile := bmpfile + help + '_';                    // day
  help := timehelp;
  bmpfile := bmpfile + ParseString(help,':') + '_';   // hour
  bmpfile := bmpfile + ParseString(help,':') + '_';   // min
  bmpfile := bmpfile + help + '.bmp';                 // sec
  CamName := Sender;
  FileName := '.\log\' + IDName + '\' + CamName + '\' + bmpfile;
  client.SendFile('.\capture.bmp',FileName);
  client.SendText('DU_SAVELOG ' + CamName + ' ' + datehelp + ' ' + timehelp + ' ' + FileName);
end;

procedure TSurveillanceController.GrabPicture(Sender: TObject);
begin
  DeviceController.Grab_Frame;
end;

procedure TSurveillanceController.Prepare;
begin
//  DeviceController.SearchDevices;
  client.Start;
  if DeviceController.cameras.SearchCameras > 0 then
  begin
    DeviceController.cameras.SelectCamera(0);
    DeviceController.cameras.SetFilename('.\capture.bmp');
  end;
end;

procedure TSurveillanceController.Start;
var i : integer;
begin
  Client.SendText('DU_MAKESUDIR ');

  with DeviceController do
  begin
    if joystick1xenabled then Client.SendText('DU_MAKESENSORFILE Temperature_Sensor_joy1x');
    if joystick1yenabled then Client.SendText('DU_MAKESENSORFILE Temperature_Sensor_joy1y');
    if joystick2xenabled then Client.SendText('DU_MAKESENSORFILE Temperature_Sensor_joy2x');
    if joystick2yenabled then Client.SendText('DU_MAKESENSORFILE Temperature_Sensor_joy2y');

    if ComPort1CTSType  <> 'Disabled' then Client.SendText('DU_MAKESENSORFILE ' + ComPort1CTSType  + '_com1CTS');
    if ComPort1DSRType  <> 'Disabled' then Client.SendText('DU_MAKESENSORFILE ' + ComPort1DSRType  + '_com1DSR');
    if ComPort1RingType <> 'Disabled' then Client.SendText('DU_MAKESENSORFILE ' + ComPort1RingType + '_com1RING');
    if ComPort1RLSDType <> 'Disabled' then Client.SendText('DU_MAKESENSORFILE ' + ComPort1RLSDType + '_com1RLDS');

    if ComPort2CTSType  <> 'Disabled' then Client.SendText('DU_MAKESENSORFILE ' + ComPort2CTSType  + '_com2CTS');
    if ComPort2DSRType  <> 'Disabled' then Client.SendText('DU_MAKESENSORFILE ' + ComPort2DSRType  + '_com2DSR');
    if ComPort2RingType <> 'Disabled' then Client.SendText('DU_MAKESENSORFILE ' + ComPort2RingType + '_com2RING');
    if ComPort2RLSDType <> 'Disabled' then Client.SendText('DU_MAKESENSORFILE ' + ComPort2RLSDType + '_com2RLDS');

    for i:=0 to DeviceController.Cameras.getCameraCount - 1 do
      Client.SendText('DU_MAKECAMDIR Camera_Cam' + IntToStr(i));
    DeviceController.StartSurvey;
    if DeviceController.Camera0Interval >= 10000 then
      CameraTimer.Interval := DeviceController.Camera0Interval*1000;
    CameraTimer.Enabled := True;
  end;
end;

procedure TSurveillanceController.Stop;
begin
   DeviceController.StopSurvey;
end;

procedure TSurveillanceController.LoadSettings(filename: string = '.\localconfig.ini');
var inifile : TIniFile;
    NumberOfDevices,NumberOfAlarmModes,i : integer;
    //j : integer;
    //SensorID : integer;
    DeviceName : string;
begin
  inifile := TIniFile.Create(filename);
  with DeviceController do
  begin
    DeviceController.joystick1xoldvalue := 23;
    DeviceController.joystick1xmin    := StrToInt(inifile.ReadString('joystick1','x-axel_min','1'));
    DeviceController.joystick1xminc   := StrToInt(inifile.ReadString('joystick1','x-axel_minc','1'));
    DeviceController.joystick1xFactor := StrToInt(inifile.ReadString('joystick1','x-axel_factor','1'));
    DeviceController.joystick1xenabled:= not(StrToBool(inifile.ReadString('joystick1','x-axel_enabled','-1')));
    DeviceController.joystick1yoldvalue := 23;
    DeviceController.joystick1ymin    := StrToInt(inifile.ReadString('joystick1','y-axel_min','1'));
    DeviceController.joystick1yminc   := StrToInt(inifile.ReadString('joystick1','y-axel_minc','1'));
    DeviceController.joystick1yFactor := StrToInt(inifile.ReadString('joystick1','y-axel_factor','1'));
    DeviceController.joystick1yenabled:= not(StrToBool(inifile.ReadString('joystick1','y-axel_enabled','-1')));

    DeviceController.joystick2xoldvalue := 23;
    DeviceController.joystick2xmin    := StrToInt(inifile.ReadString('joystick2','x-axel_min','1'));
    DeviceController.joystick2xminc   := StrToInt(inifile.ReadString('joystick2','x-axel_minc','1'));
    DeviceController.joystick2xFactor := StrToInt(inifile.ReadString('joystick2','x-axel_factor','1'));
    DeviceController.joystick2xenabled:= not(StrToBool(inifile.ReadString('joystick2','x-axel_enabled','-1')));
    DeviceController.joystick2yoldvalue := 23;
    DeviceController.joystick2ymin    := StrToInt(inifile.ReadString('joystick2','y-axel_min','1'));
    DeviceController.joystick2yminc   := StrToInt(inifile.ReadString('joystick2','y-axel_minc','1'));
    DeviceController.joystick2yFactor := StrToInt(inifile.ReadString('joystick2','y-axel_factor','1'));
    DeviceController.joystick2yenabled:= not(StrToBool(inifile.ReadString('joystick2','y-axel_enabled','-1')));

    DeviceController.ComPort1CTSType := inifile.ReadString('comport1','CTS','Disabled');
    if ComPort1CTSType       = 'Motion Detector' then ComPort1CTSType  := 'Motion_Detector'
    else if ComPort1CTSType  = 'Wet/Dry Sensor'  then ComPort1CTSType  := 'Wet_Dry_Sensor'
    else if ComPort1CTSType  = 'On/Off Sensor'   then ComPort1CTSType  := 'On_Off_Sensor'
    else ComPort1CTSType  := 'Disabled';

    DeviceController.ComPort1DSRType := inifile.ReadString('comport1','DSR','Disabled');
    if ComPort1DSRType       = 'Motion Detector' then ComPort1DSRType  := 'Motion_Detector'
    else if ComPort1DSRType  = 'Wet/Dry Sensor'  then ComPort1DSRType  := 'Wet_Dry_Sensor'
    else if ComPort1DSRType  = 'On/Off Sensor'   then ComPort1DSRType  := 'On_Off_Sensor'
    else ComPort1DSRType  := 'Disabled';

    DeviceController.ComPort1RingType := inifile.ReadString('comport1','Ring','Disabled');
    if ComPort1RingType      = 'Motion Detector' then ComPort1RingType := 'Motion_Detector'
    else if ComPort1RingType = 'Wet/Dry Sensor'  then ComPort1RingType := 'Wet_Dry_Sensor'
    else if ComPort1RingType = 'On/Off Sensor'   then ComPort1RingType := 'On_Off_Sensor'
    else ComPort1RingType := 'Disabled';

    DeviceController.ComPort1RLSDType := inifile.ReadString('comport1','RLSD','Disabled');
    if ComPort1RLSDType      = 'Motion Detector' then ComPort1RLSDType := 'Motion_Detector'
    else if ComPort1RLSDType = 'Wet/Dry Sensor'  then ComPort1RLSDType := 'Wet_Dry_Sensor'
    else if ComPort1RLSDType = 'On/Off Sensor'   then ComPort1RLSDType := 'On_Off_Sensor'
    else ComPort1RLSDType := 'Disabled';

    DeviceController.ComPort1RTSType := inifile.ReadString('comport1','RTS','Disabled');
    if ComPort1RTSType       = 'Local Alarm'     then ComPort1RTSType  := 'Local_Alarm'
    else ComPort1RTSType  := 'Disabled';

    DeviceController.ComPort1CTSStatusDefault := StrToBool(inifile.ReadString('comport1','CTSInvert','-1'));
    DeviceController.ComPort1DSRStatusDefault := StrToBool(inifile.ReadString('comport1','DSRInvert','-1'));
    DeviceController.ComPort1RingStatusDefault := StrToBool(inifile.ReadString('comport1','RingInvert','-1'));
    DeviceController.ComPort1RLSDStatusDefault := StrToBool(inifile.ReadString('comport1','RLSDInvert','-1'));
    DeviceController.ComPort1RTSStatusDefault := StrToBool(inifile.ReadString('comport1','RTSInvert','-1'));

    DeviceController.ComPort2CTSType := inifile.ReadString('comport1','CTS','Disabled');
    if ComPort2CTSType  = 'Motion Detector' then ComPort2CTSType  := 'Motion_Detector'
    else if ComPort2CTSType  = 'Wet/Dry Sensor'  then ComPort2CTSType  := 'Wet_Dry_Sensor'
    else if ComPort2CTSType  = 'On/Off Sensor'   then ComPort2CTSType  := 'On_Off_Sensor'
    else ComPort2CTSType  := 'Disabled';

    DeviceController.ComPort2DSRType := inifile.ReadString('comport1','DSR','Disabled');
    if ComPort2DSRType  = 'Motion Detector' then ComPort2DSRType  := 'Motion_Detector'
    else if ComPort2DSRType  = 'Wet/Dry Sensor'  then ComPort2DSRType  := 'Wet_Dry_Sensor'
    else if ComPort2DSRType  = 'On/Off Sensor'   then ComPort2DSRType  := 'On_Off_Sensor'
    else ComPort2DSRType  := 'Disabled';

    DeviceController.ComPort2RingType := inifile.ReadString('comport1','Ring','Disabled');
    if ComPort2RingType = 'Motion Detector' then ComPort2RingType := 'Motion_Detector'
    else if ComPort2RingType = 'Wet/Dry Sensor'  then ComPort2RingType := 'Wet_Dry_Sensor'
    else if ComPort2RingType = 'On/Off Sensor'   then ComPort2RingType := 'On_Off_Sensor'
    else ComPort2RingType := 'Disabled';

    DeviceController.ComPort2RLSDType := inifile.ReadString('comport1','RLSD','Disabled');
    if ComPort2RLSDType = 'Motion Detector' then ComPort2RLSDType := 'Motion_Detector'
    else if ComPort2RLSDType = 'Wet/Dry Sensor'  then ComPort2RLSDType := 'Wet_Dry_Sensor'
    else if ComPort2RLSDType = 'On/Off Sensor'   then ComPort2RLSDType := 'On_Off_Sensor'
    else ComPort2RLSDType := 'Disabled';

    DeviceController.ComPort2RTSType := inifile.ReadString('comport1','RTS','Disabled');
    if ComPort2RTSType  = 'Local Alarm'     then ComPort2RTSType  := 'Local_Alarm'
    else ComPort2RTSType  := 'Disabled';

    DeviceController.ComPort2CTSStatusDefault := StrToBool(inifile.ReadString('comport2','CTSInvert','-1'));
    DeviceController.ComPort2DSRStatusDefault := StrToBool(inifile.ReadString('comport2','DSRInvert','-1'));
    DeviceController.ComPort2RingStatusDefault := StrToBool(inifile.ReadString('comport2','RingInvert','-1'));
    DeviceController.ComPort2RLSDStatusDefault := StrToBool(inifile.ReadString('comport2','RLSDInvert','-1'));
    DeviceController.ComPort2RTSStatusDefault := StrToBool(inifile.ReadString('comport2','RTSInvert','-1'));
  end; // end with

  Client.HostIP := inifile.ReadString('dataunit','dataunitip','');
  Client.Port := StrToInt(inifile.ReadString('dataunit','dataunitport',''));
  IDName := inifile.ReadString('surveillanceunit','surveillanceunit','');

  inifile.Free;
  inifile := TIniFile.Create('.\' + IDName + '.ini');

//SensorID  Sensor
//   0    ComPort1CTS
//   1    ComPort1DSR
//   2    ComPort1Ring
//   3    ComPort1RLSD
//   4    ComPort1RTS
//   5    ComPort2CTS
//   6    ComPort2DSR
//   7    ComPort2Ring
//   8    ComPort2RLSD
//   9    ComPort2RTS
//   10   Joystick1x
//   11   Joystick1y
//   12   Joystick2x
//   13   Joystick2y
//   14   Camera 1
//   15   Camera 2
//   16   Camera 3
//   17   Camera 4

  NumberOfDevices := StrToInt(inifile.ReadString('General','NumberOf','0'));

  for i:=0 to NumberOfDevices-1 do
  begin
    if not(inifile.SectionExists('device' + IntToStr(i))) then continue;
//    SensorID := inifile.ReadString('device' + IntToStr(i),'SensorID','0');
    DeviceName := inifile.ReadString('device' + IntToStr(i),'DeviceName','');
    if Devicename = 'com1CTS' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.Comport1CTSInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Devices[0].LimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Devices[0].LimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Devices[0].LimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Devices[0].LimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');

        Comport1CTSLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Comport1CTSLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Comport1CTSLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Comport1CTSLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'com1DSR' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.Comport1DSRInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Comport1DSRLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Comport1DSRLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Comport1DSRLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Comport1DSRLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'com1RING' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.Comport1RingInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Comport1RingLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Comport1RingLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Comport1RingLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Comport1RingLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'com1RLSD' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.Comport1RLSDInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Comport1RLSDLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Comport1RLSDLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Comport1RLSDLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Comport1RLSDLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'com1RTS' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.Comport1RTSInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Comport1RTSLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Comport1RTSLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Comport1RTSLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Comport1RTSLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'com2CTS' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.Comport2CTSInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Comport2CTSLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Comport2CTSLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Comport2CTSLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Comport2CTSLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'com2DSR' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.Comport2DSRInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Comport2DSRLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Comport2DSRLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Comport2DSRLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Comport2DSRLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'com2RING' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.Comport2RingInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Comport2RingLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Comport2RingLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Comport2RingLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Comport2RingLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'com2RLSD' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.Comport2RLSDInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Comport2RLSDLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Comport2RLSDLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Comport2RLSDLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Comport2RLSDLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'com2RTS' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.Comport2RTSInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Comport2RTSLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Comport2RTSLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Comport2RTSLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Comport2RTSLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'joy1x' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.joystick1xInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Joystick1xLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Joystick1xLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Joystick1xLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Joystick1xLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'joy1y' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.joystick1yInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Joystick1yLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Joystick1yLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Joystick1yLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Joystick1yLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'joy2x' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.joystick2xInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Joystick2xLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Joystick2xLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Joystick2xLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Joystick2xLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'joy2y' then
    begin
      //inifile.ReadString('device' + IntToStr(i),'Type','');
      DeviceController.joystick2yInterval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
      NumberOfAlarmModes := StrToInt(inifile.ReadString('device' + IntToStr(i),'AlarmModes','0'));
      if NumberOfAlarmModes >= 1 then
      //for j:=1 to NumberOfAlarmModes do
      begin
        if not(inifile.SectionExists(DeviceName + 'Limits' + IntToStr(i))) then continue;
        //inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'priority','1');
        Joystick2yLimitsStart := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Start','0:00'));
        Joystick2yLimitsEnd := StrToTime(inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'End','23:59'));
        Joystick2yLimitsMin := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Min','Off');
        Joystick2yLimitsMax := inifile.ReadString(DeviceName + 'Limits' + IntToStr(i),'Max','Off');
      end;
    end
    else if Devicename = 'cam0' then
    begin
      DeviceController.Camera0Interval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
    end
    else if Devicename = 'cam1' then
    begin
      DeviceController.Camera1Interval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
    end
    else if Devicename = 'cam2' then
    begin
      DeviceController.Camera2Interval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
    end
    else if Devicename = 'cam3' then
    begin
      DeviceController.Camera3Interval := StrToInt(inifile.ReadString('device' + IntToStr(i),'CaptionInterval','60'));
    end;

  end;  // for i:=0 to NumberOfDevices-1

  inifile.Free;
end;

procedure TSurveillanceController.SetIDName(NewName: string);
begin
  FIDName := NewName;
  Client.Name := NewName;
end;

end.
